Re: [C++ patch] do not make extern inlines as needed when not optimizing

2014-12-15 Thread Jason Merrill

On 12/14/2014 06:14 PM, Jan Hubicka wrote:

DECL_EXTERNAL (decl) && !DECL_NOT_REALLY_EXTERN (decl)


This is DECL_REALLY_EXTERN.  OK with that change (in both places).

Jason



[C++ patch] do not make extern inlines as needed when not optimizing

2014-12-14 Thread Jan Hubicka
Hi,
while looking into PR ipa/61324 I noticed that extern inline functions are
produced by C++ FE even at -O0.  This seems like considerable waste especially
because these may trigger instantiations.

Bootstrepped/regtested x86_64-linux, OK?

Honza

* decl2.c (decl_needed_p): When not optimizing, extern decls are not
needed.
Index: cp/decl2.c
===
--- cp/decl2.c  (revision 218701)
+++ cp/decl2.c  (working copy)
@@ -1996,19 +1996,31 @@ decl_needed_p (tree decl)
  COMDAT until that point.  */
   gcc_assert (at_eof);
 
   /* All entities with external linkage that are not COMDAT should be
  emitted; they may be referred to from other object files.  */
-  if (TREE_PUBLIC (decl) && !DECL_COMDAT (decl))
+  if (TREE_PUBLIC (decl) && !DECL_COMDAT (decl)
+  && (!DECL_EXTERNAL (decl) || DECL_NOT_REALLY_EXTERN (decl)))
 return true;
-  /* If this entity was used, let the back end see it; it will decide
- whether or not to emit it into the object file.  */
-  if (TREE_USED (decl))
-  return true;
   /* Functions marked "dllexport" must be emitted so that they are
  visible to other DLLs.  */
   if (flag_keep_inline_dllexport
   && lookup_attribute ("dllexport", DECL_ATTRIBUTES (decl)))
 return true;
+
+  /* When not optimizing, do not bother to produce definitions for extern
+ symbols.  */
+  if (DECL_EXTERNAL (decl) && !DECL_NOT_REALLY_EXTERN (decl)
+  && ((TREE_CODE (decl) != FUNCTION_DECL
+  && !optimize)
+ || (TREE_CODE (decl) == FUNCTION_DECL
+ && !opt_for_fn (decl, optimize)))
+  && !lookup_attribute ("always_inline", decl))
+return false;
+
+  /* If this entity was used, let the back end see it; it will decide
+ whether or not to emit it into the object file.  */
+  if (TREE_USED (decl))
+  return true;
   /* Virtual functions might be needed for devirtualization.  */
   if (flag_devirtualize
   && TREE_CODE (decl) == FUNCTION_DECL
@@ -1996,19 +1996,31 @@ decl_needed_p (tree decl)
   gcc_assert (at_eof);
 
-  /* All entities with external linkage that are not COMDAT should be
+  /* All entities with external linkage that are not COMDAT/EXTERN should be
  emitted; they may be referred to from other object files.  */